<template>
	<div class="super-flow-base-demo">
		<super-flow ref="superFlow" :node-list="nodeList" :link-list="linkList" :origin="origin" :graph-menu="graphMenuList" :node-menu="nodeMenuList" :link-menu="linkMenuList" :link-desc="linkDesc" :link-base-style="linkBaseStyle" :draggable="draggable" :linkAddable="linkAddable" :linkEditable="linkEditable">
			<template v-slot:node="{meta}">
				<div :class="`flow-node flow-node-${meta.prop}`">
					<header class="ellipsis" :title="meta.name">
						{{ meta.name }}
					</header>
					<section :title="meta.desc">
						{{ meta.desc }}
					</section>
				</div>
			</template>
		</super-flow>

		<el-dialog :title="drawerConf.title" :visible.sync="drawerConf.visible" :close-on-click-modal="false" width="500px">
			<el-form @keyup.native.enter="settingSubmit" @submit.native.prevent v-show="drawerConf.type === drawerType.node" ref="nodeSetting" :model="nodeSetting">
				<el-form-item label="节点名称" prop="name">
					<el-input v-model="nodeSetting.name" placeholder="请输入节点名称" maxlength="30">
					</el-input>
				</el-form-item>
				<el-form-item label="节点描述" prop="desc">
					<el-input v-model="nodeSetting.desc" placeholder="请输入节点描述" maxlength="30">
					</el-input>
				</el-form-item>
			</el-form>
			<el-form @keyup.native.enter="settingSubmit" @submit.native.prevent v-show="drawerConf.type === drawerType.link" ref="linkSetting" :model="linkSetting">
				<el-form-item label="连线描述" prop="desc">
					<el-input v-model="linkSetting.desc" placeholder="请输入连线描述">
					</el-input>
				</el-form-item>
			</el-form>
			<span slot="footer" class="dialog-footer">
				<el-button @click="drawerConf.cancel">
					取 消
				</el-button>
				<el-button type="primary" @click="settingSubmit">
					确 定
				</el-button>
			</span>
		</el-dialog>

	</div>
</template>

<script>
import SuperFlow from 'vue-super-flow'
import 'vue-super-flow/lib/index.css'
const drawerType = {
	node: 0,
	link: 1
}
const graphMenuList = [
	[
		{
			label: '系统派单',
			disable(graph) {
				return !!graph.nodeList.find(node => node.meta.prop === 'start')
			},
			selected: (graph, coordinate) => {
				const start = graph.nodeList.find(node => node.meta.prop === 'start')
				if (!start) {
					graph.addNode({
						width: 100,
						height: 70,
						coordinate: coordinate,
						meta: {
							prop: 'start',
							name: '系统派单'
						}
					})
				}
			}
		},
		{
			label: '未完成',
			disable: false,
			selected: (graph, coordinate) => {
				graph.addNode({
					width: 100,
					height: 70,
					coordinate: coordinate,
					meta: {
						prop: 'condition',
						name: '未完成'
					}
				})
			}
		},
		{
			label: '等待中',
			disable: false,
			selected: (graph, coordinate) => {
				graph.addNode({
					width: 100,
					height: 70,
					coordinate: coordinate,
					meta: {
						prop: 'approval',
						name: '等待中'
					}
				})
			}
		},
		{
			label: '已完成',
			disable: false,
			selected: (graph, coordinate) => {
				graph.addNode({
					width: 100,
					height: 70,
					coordinate: coordinate,
					meta: {
						prop: 'cc',
						name: '已完成'
					}
				})
			}
		},
	],
	[
		{
			label: '打印数据',
			selected: (graph, coordinate) => {
				console.log(JSON.stringify(graph.toJSON(), null, 2))
			}
		},
		{
			label: '全选',
			selected: (graph, coordinate) => {
				graph.selectAll()
			}
		}
	]
]
const taskStateMap = ['cc', 'approval', 'condition']
const actionTypeMap = ['派发', '分派', '转派', '平移']
export default {
	components: { SuperFlow },
	props: {
		isEdit: {
			type: Boolean,
			default: false
		}
	},
	data() {
		return {
			draggable: false,
			linkAddable: false,
			linkEditable: false,
			drawerType,
			drawerConf: {
				title: '',
				visible: false,
				type: null,
				info: null,
				open: (type, info) => {
					const conf = this.drawerConf
					conf.visible = true
					conf.type = type
					conf.info = info
					if (conf.type === drawerType.node) {
						conf.title = '节点'
						if (this.$refs.nodeSetting) this.$refs.nodeSetting.resetFields()
						this.$set(this.nodeSetting, 'name', info.meta.name)
						this.$set(this.nodeSetting, 'desc', info.meta.desc)
					} else {
						conf.title = '连线'
						if (this.$refs.linkSetting) this.$refs.linkSetting.resetFields()
						this.$set(this.linkSetting, 'desc', info.meta ? info.meta.desc : '')
					}
				},
				cancel: () => {
					this.drawerConf.visible = false
					if (this.drawerConf.type === drawerType.node) {
						this.$refs.nodeSetting.clearValidate()
					} else {
						this.$refs.linkSetting.clearValidate()
					}
				}
			},
			linkBaseStyle: {
				color: '#999999',           // line 颜色
				hover: '#55abfc',           // line hover 的颜色
				textColor: '#666666',       // line 描述文字颜色
				textHover: '#55abfc',       // line 描述文字 hover 颜色
				font: '14px Arial',         // line 描述文字 字体设置 参考 canvas font
				dotted: true,              // 是否是虚线
				lineDash: [4, 4],           // 虚线时生效
				background: 'rgba(255,255,255,0.6)'    // 描述文字背景色
			},
			linkSetting: {
				desc: ''
			},
			nodeSetting: {
				name: '',
				desc: ''
			},
			origin: [0, 0],
			nodeList: [],
			linkList: [],
			graphMenuList: [],
			nodeMenuList: [
				[
					{
						label: '编辑',
						selected: (node, coordinate) => {
							this.drawerConf.open(drawerType.node, node)
						}
					}
				],
				[
					{
						label: '删除',
						disable: false,
						hidden(node) {
							return node.meta.prop === 'start'
						},
						selected(node, coordinate) {
							node.remove()
						}
					}
				],
			],
			linkMenuList: [
				[
					{
						label: '编辑',
						disable: false,
						selected: (link, coordinate) => {
							this.drawerConf.open(drawerType.link, link)
						}
					}
				],
				[
					{
						label: '删除',
						disable: false,
						selected: (link, coordinate) => {
							link.remove()
						}
					}
				],
			]
		}
	},
	created() {
		this.draggable = this.isEdit
		this.linkAddable = this.isEdit
		this.linkEditable = this.isEdit
		if (this.isEdit) {
			this.graphMenuList = graphMenuList
		} else {
			this.graphMenuList = []
		}

		let resData = [{
			id: 1,//id
			parentId: null,//父节点id  如果没有则为null
			title: "工单生成",//标题
			desc: "系统自动派单",//具体信息
			taskState: 0,//任务状态 0:'已完成', 1:'等待中', 2:'未完成'
			actionType: null//动作类型  如果没有则为null 0:'派发', 1:'分派', 2:'转派', 3:'平移'
		}, {
			id: 2,//id
			parentId: 1,//父节点id  如果没有则为null
			title: "区管理员",//标题
			desc: "李某某",//具体信息
			taskState: 1,//任务状态
			actionType: 0//动作类型
		}, {
			id: 3,//id
			parentId: 2,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "王某某",//具体信息
			taskState: 0,//任务状态
			actionType: 1//动作类型
		}, {
			id: 4,//id
			parentId: 2,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "王某某",//具体信息
			taskState: 0,//任务状态
			actionType: 1//动作类型
		}, {
			id: 6,//id
			parentId: 2,//父节点id  如果没有则为null
			title: "蓄电池维护班组",//标题
			desc: "李某某",//具体信息
			taskState: 1,//任务状态
			actionType: 2//动作类型
		}, {
			id: 55,//id
			parentId: 6,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "王某某",//具体信息
			taskState: 0,//任务状态
			actionType: 3//动作类型
		}, {
			id: 8,//id
			parentId: 6,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "赵某某",//具体信息
			taskState: 0,//任务状态
			actionType: 1//动作类型
		}, {
			id: 9,//id
			parentId: 6,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "赵某某",//具体信息
			taskState: 2,//任务状态
			actionType: 1//动作类型
		}, {
			id: 7,//id
			parentId: 2,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "王某某",//具体信息
			taskState: 0,//任务状态
			actionType: 3//动作类型
		}, {
			id: 5,//id
			parentId: 2,//父节点id  如果没有则为null
			title: "操作员",//标题
			desc: "赵某某",//具体信息
			taskState: 0,//任务状态
			actionType: 1//动作类型
		},]
		setTimeout(() => {
			const nodeList = [], linkList = []
			let inParentIndex = 0 //记录当前子节点对应父节点索引值
			resData.map((item, i) => {
				let nodeObj = {}, linkObj = {}
				if (i == 0) {
					nodeObj = {
						id: item.id,
						width: 100,
						height: 70,
						coordinate: [100, 25],
						meta: {
							prop: taskStateMap[item.taskState],
							name: item.title,
							desc: item.desc,
						},
						actionType: item.actionType
					}
				} else {
					let parentNode = nodeList.find(node => node.id === item.parentId)
					nodeObj = {
						id: item.id,
						width: 100,
						height: 70,
						coordinate: [100, 25],
						meta: {
							prop: taskStateMap[item.taskState],
							name: item.title,
							desc: item.desc,
						},
						actionType: item.actionType
					}
					linkObj = {
						id: "linkWFnrfOl5JnlGXeYf",
						startId: parentNode.id,
						endId: item.id,
						startAt: [
							50,
							70
						],
						endAt: [
							50,
							0
						],
						meta: {
							desc: actionTypeMap[item.actionType]
						}
					}
					switch (item.actionType) {
						case 0://派发
							nodeObj.coordinate = [parentNode.coordinate[0], parentNode.coordinate[1] + 180]
							linkObj.startAt = [50, 70]
							linkObj.endAt = [50, 0]
							break;
						case 1://'分派'
							if (this[`parentNode${parentNode.id}-${item.actionType}`]) {
								this[`inParentIndex-${parentNode.id}-${item.actionType}`]++
							} else {
								this[`parentNode${parentNode.id}-${item.actionType}`] = parentNode
								this[`inParentIndex-${parentNode.id}-${item.actionType}`] = 0
							}
							nodeObj.coordinate = [parentNode.coordinate[0] + 130 + 130 * this[`inParentIndex-${parentNode.id}-${item.actionType}`], parentNode.coordinate[1] + 150]
							linkObj.startAt = [50, 70]
							linkObj.endAt = [50, 0]
							break;
						case 2://'转派'
							nodeObj.coordinate = [parentNode.coordinate[0], parentNode.coordinate[1] + 250]
							linkObj.startAt = [50, 70]
							linkObj.endAt = [50, 0]
							break;
						case 3://'平移'
							if (this[`parentNode${parentNode.id}-${item.actionType}`]) {
								this[`inParentIndex-${parentNode.id}-${item.actionType}`]++
							} else {
								this[`parentNode${parentNode.id}-${item.actionType}`] = parentNode
								this[`inParentIndex-${parentNode.id}-${item.actionType}`] = 0
							}
							let parentAssLength = resData.filter(function (obj) {
								return (obj.parentId == item.parentId && obj.actionType == 1);
							}).length //计算同父节点拥有分派的数量
							let AssDistance = parentNode.coordinate[0] + 130 * parentAssLength + 30;//计算同父节点拥有分派的平移位置
							nodeObj.coordinate = [parentNode.coordinate[0] + AssDistance + 130 * this[`inParentIndex-${parentNode.id}-${item.actionType}`], parentNode.coordinate[1], parentNode.coordinate[1]]
							linkObj.startAt = [100, 35]
							linkObj.endAt = [0, 35]
							break;
					}
				}
				nodeList.push(nodeObj)
				linkList.push(linkObj)
			})
			this.nodeList = nodeList
			this.linkList = linkList
		}, 100)
	},
	mounted() {

	},
	methods: {
		linkDesc(link) {
			return link.meta ? link.meta.desc : ''
		},
		settingSubmit() {
			const conf = this.drawerConf
			if (this.drawerConf.type === drawerType.node) {
				if (!conf.info.meta) conf.info.meta = {}
				Object.keys(this.nodeSetting).forEach(key => {
					this.$set(conf.info.meta, key, this.nodeSetting[key])
				})
				this.$refs.nodeSetting.resetFields()
			} else {
				if (!conf.info.meta) conf.info.meta = {}
				Object.keys(this.linkSetting).forEach(key => {
					this.$set(conf.info.meta, key, this.linkSetting[key])
				})
				this.$refs.linkSetting.resetFields()
			}
			conf.visible = false
		}
	}
}
</script>

<style lang="less">
.ellipsis {
	white-space: nowrap;
	text-overflow: ellipsis;
	overflow: hidden;
	word-wrap: break-word;
}

.super-flow-base-demo {
	width: 100%;
	height: 100%;
	background-color: #f5f5f5;
	overflow: scroll;

	.super-flow {
		width: 8000px;
		height: 8000px;
	}

	.super-flow__node {
		border: 1px solid #e4e4e4;
		border-radius: 4px;
		overflow: hidden;
		.flow-node {
			> header {
				font-size: 14px;
				height: 32px;
				line-height: 32px;
				padding: 0 12px;
				color: #ffffff;
				text-align: center;
			}

			> section {
				text-align: center;
				line-height: 20px;
				padding: 6px 12px;
				word-break: break-all;
				font-size: 14px;
				color: #666;
				text-overflow: -o-ellipsis-lastline;
				overflow: hidden;
				text-overflow: ellipsis;
				display: -webkit-box;
				-webkit-line-clamp: 2;
				line-clamp: 2;
				-webkit-box-orient: vertical;
			}

			&.flow-node-start {
				> header {
					background-color: #55abfc;
				}
			}

			&.flow-node-condition {
				> header {
					background-color: #bc1d16;
				}
			}

			&.flow-node-approval {
				> header {
					background-color: rgba(188, 181, 58, 0.76);
				}
			}

			&.flow-node-cc {
				> header {
					background-color: #30b95c;
				}
			}

			&.flow-node-end {
				> header {
					height: 50px;
					line-height: 50px;
					background-color: rgb(0, 0, 0);
				}
			}
		}
	}
}
</style>